@@ -153,7 +153,7 @@ class FormatDecisionState(object):
         format_token.Subtype.DEFAULT_OR_NAMED_ASSIGN_ARG_LIST in
         current.subtypes):
       if (previous.value not in {'(', '=', '*', '**'} and
-          current.value not in '=,)'): 
+          current.value not in '=,)'):
         opening = _GetOpeningParen(current)
         if opening:
           arglist_length = (
@@ -171,7 +171,8 @@ class FormatDecisionState(object):
 
     if (format_token.Subtype.COMP_FOR in current.subtypes and
         format_token.Subtype.COMP_FOR not in previous.subtypes):
-      length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_FOR)
+      length = _GetLengthOfSubtype(current, format_token.Subtype.COMP_FOR,
+                                   format_token.Subtype.COMP_IF)
       if length + self.column > column_limit:
         return True
 
@@ -203,6 +204,10 @@ class FormatDecisionState(object):
             # up the formatting.
             return True
 
+    if (style.Get('SPLIT_BEFORE_BITWISE_OPERATOR') and
+        current.value in '&|' and previous.lineno < current.lineno):
+      return True
+
     if (current.is_comment and
         previous.lineno < current.lineno - current.value.count('\n')):
       # If a comment comes in the middle of an unwrapped line (like an if
@@ -441,9 +446,10 @@ def _IsDictionaryValue(token):
   return False
 
 
-def _GetLengthOfSubtype(token, subtype):
+def _GetLengthOfSubtype(token, subtype, exclude=None):
   current = token
-  while current.next_token and subtype in current.subtypes:
+  while (current.next_token and subtype in current.subtypes and
+         (exclude is None or exclude not in current.subtypes)):
     current = current.next_token
   return current.total_length - token.total_length + 1
 
